"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const dist_1 = require("@now/build-utils/dist");
const child_process_1 = require("child_process");
const path_1 = __importDefault(require("path"));
const PHP_PKG = path_1.default.resolve(require.resolve('@now-bref/lib'), '../..');
const PHP_BIN_DIR = path_1.default.join(PHP_PKG, "native/bref/bin");
const PHP_MODULES_DIR = path_1.default.join(PHP_BIN_DIR, "native/bref/lib/php/extensions/no-debug-zts-20180731");
const PHP_LIB_DIR = path_1.default.join(PHP_PKG, "native/bref/lib");
const PHP_LIB64_DIR = path_1.default.join(PHP_PKG, "native/bref/lib64");
const COMPOSER_BIN = path_1.default.join(PHP_PKG, "native/bin/composer");
async function getIncludedFiles({ files, entrypoint, workPath, config, meta }) {
    // Download all files to workPath
    const downloadedFiles = await dist_1.download(files, workPath, meta);
    let includedFiles = {};
    if (config && config.includeFiles) {
        // Find files for each glob
        for (const pattern of config.includeFiles) {
            const matchedFiles = await dist_1.glob(pattern, workPath);
            Object.assign(includedFiles, matchedFiles);
        }
        // explicit and always include the entrypoint
        Object.assign(includedFiles, {
            [entrypoint]: files[entrypoint],
        });
    }
    else {
        // Backwards compatibility
        includedFiles = downloadedFiles;
    }
    return includedFiles;
}
exports.getIncludedFiles = getIncludedFiles;
async function getComposerFiles(workPath) {
    console.log('🐘 Installing Composer deps.');
    // Install composer dependencies
    await runComposerInstall(workPath);
    console.log('🐘 Installing Composer deps OK.');
    return await dist_1.glob('vendor/**', workPath);
}
exports.getComposerFiles = getComposerFiles;
async function runComposerInstall(cwd) {
    // @todo think about allow to pass custom commands here
    await runPhp(cwd, [
        COMPOSER_BIN,
        'install',
        '--profile',
        '--no-dev',
        '--no-interaction',
        '--no-scripts',
        '--ignore-platform-reqs',
        '--no-progress'
    ], { stdio: 'inherit' });
}
async function runPhp(cwd, args, opts = {}) {
    try {
        await spawnAsync('php', [`-dextension_dir=${PHP_MODULES_DIR}`, ...args], cwd, {
            ...opts,
            ...{
                env: {
                    ...process.env,
                    ...{
                        COMPOSER_HOME: '/tmp',
                        PATH: `${PHP_BIN_DIR}:${process.env.PATH}`,
                        LD_LIBRARY_PATH: `${PHP_LIB_DIR}:${PHP_LIB64_DIR}:/usr/lib64:/lib64:${process.env.LD_LIBRARY_PATH}`
                    }
                }
            }
        });
    }
    catch (e) {
        console.error(e);
        process.exit(1);
    }
}
function spawnAsync(command, args, cwd, opts = {}) {
    return new Promise((resolve, reject) => {
        const child = child_process_1.spawn(command, args, {
            stdio: "ignore",
            cwd,
            ...opts
        });
        child.on('error', reject);
        child.on('exit', (code, signal) => {
            if (code === 0) {
                resolve();
            }
            else {
                reject(new Error(`Exited with ${code || signal}`));
            }
        });
    });
}
